import configparser as _configparser
from _datetime import datetime as _datetime
import os as _os
import sys
#sys.path.append('C:\\Users\\coliveira\\OneDrive\\Coding\\Python\\MFToolbox\\')
from mftoolbox import encoding

class Build:
    """
    O controle de build é feito através da data e hora em que o arquivo do Magic Formula foi salvo pela última vez.
    Se a data do arquivo for mais recente do que está registrado no controle de build, o número do build
        será incrementado em 1.
    Se o arquivo de build ainda não existir, será criado com o nome da versão atual (hardcoded), build=0, data de
        atualização = data que o arquivo do Magic Formula foi salvo pela última vez.
    Se o arquivo de build existir e já houver uma entrada para o nome da versão atual, compara o horário da última
        gravação do Magic Formula. Se a última gravação for mais recente do que o registro no controle de build,
        este será incrementado em 1.
    Se o arquivo de build existir e não houver uma entrada para o nome da versão atual, esta será criada com
        o nome da versão atual (hardcoded), build=0, data de atualização = data que o arquivo do Magic Formula
        foi salvo pela última vez.
    07/10/19: controle para definir build diferente por data apenas ou por data e hora. 0 para data e 1 para data e hora
    """
    def __init__(self, VersaoAtual, FileName, Controle):

        self.str_versao_atual = VersaoAtual
        self.str_nome_arquivo_build = 'BuildHistory.txt'
        self.str_encoding = encoding.encoding()
        self.str_nome_arquivo_chamada = FileName

        # data em que o arquivo atual foi salvo
        self.dtt_ultima_atualizacao = _os.stat(_os.path.basename(FileName)).st_mtime

        # cria se não existe o arquivo com os dados do build atual, senão, abre o arquivo existente
        if not _os.path.isfile(self.str_nome_arquivo_build):
            dic_build_content = _configparser.ConfigParser()
            dic_build_content[self.str_versao_atual] = {'build': '0',
                                                   'atualização': str(self.dtt_ultima_atualizacao),
                                                   'data': _datetime.fromtimestamp(
                                                       self.dtt_ultima_atualizacao).strftime('%d-%m-%Y %H:%M:%S')}

            with open(self.str_nome_arquivo_build, 'w', encoding=self.str_encoding) as fil_build:
                dic_build_content.write(fil_build)
                fil_build.close()
        else:
            dic_build_content = _configparser.ConfigParser()
            dic_build_content.read(self.str_nome_arquivo_build, encoding=self.str_encoding)

        # se não houver entrada para a versão atual, cria (anexando ao arquivo existente)
        # Se houver, verifica se os dados do build são iguais
        try:
            self.int_numero_build = dic_build_content.getint(self.str_versao_atual, 'build')
            flt_atualizacao = dic_build_content.getfloat(self.str_versao_atual, "atualização")
            self.str_data_build = _datetime.fromtimestamp(self.dtt_ultima_atualizacao).strftime('%d-%m-%Y %H:%M:%S')

            # se a data da última revisão do arquivo for diferente da informação do arquivo de build, aumenta o número do build
            # e atualiza o arquivo de controle de build
            if (self.dtt_ultima_atualizacao != flt_atualizacao and Controle == 1) or (_datetime.fromtimestamp(self.dtt_ultima_atualizacao).strftime('%d-%m-%Y') != _datetime.fromtimestamp(flt_atualizacao).strftime('%d-%m-%Y') and Controle == 0):
                self.int_numero_build = self.int_numero_build + 1
                dic_build_content.set(self.str_versao_atual, 'build', str(self.int_numero_build))
                dic_build_content.set(self.str_versao_atual, 'atualização', str(self.dtt_ultima_atualizacao))
                dic_build_content.set(self.str_versao_atual, 'data',
                                      _datetime.fromtimestamp(self.dtt_ultima_atualizacao).strftime(
                                          '%d-%m-%Y %H:%M:%S'))
                # sobrescreve o arquivo
                with open(self.str_nome_arquivo_build, 'w+', encoding=self.str_encoding) as fil_build:
                    dic_build_content.write(fil_build)
                fil_build.close()
        except:
            self.int_numero_build = 0
            dic_build_content = _configparser.ConfigParser()
            dic_build_content[self.str_versao_atual] = {'build': self.int_numero_build,
                                                   'atualização': str(self.dtt_ultima_atualizacao),
                                                   'data': _datetime.fromtimestamp(
                                                       self.dtt_ultima_atualizacao).strftime('%d-%m-%Y %H:%M:%S')}
            # adiciona ao que já está no arquivo
            with open(self.str_nome_arquivo_build, 'a', encoding=self.str_encoding) as fil_build:
                dic_build_content.write(fil_build)
            fil_build.close()

        self.str_ultima_atualizacao = _datetime.fromtimestamp(self.dtt_ultima_atualizacao).strftime(
            '%d-%m-%Y %H:%M:%S')
        self.txt_int_numero_build = str(self.int_numero_build)


class proventos:

    #
    # Define a classe Proventos
    # Autor: Celso Oliveira (c.oliveira@live.com)
    # Controle de versão:
    #
    #   1.0, 16/Ago/18, criação.
    #
    # Documentação de apoio:
    #
    #   http://catalogo.governoeletronico.gov.br/arquivos/Documentos/WS_SGS_BCB.pdf - descrição do uso do Webservice
    #   http://python-zeep.readthedocs.io/en/master/ - zeep, biblioteca para trabalhar com webservices
    #   http://blog.tiagocrizanto.com/configuracoes-do-webservice-do-banco-central-cotacoes-diversas/

    def __init__(self, arg_idioma, arg_cd_acao, arg_data, arg_valor, arg_tipo, arg_referencia):
        self.idioma = arg_idioma
        self.cd_acao = arg_cd_acao
        self.arg_data = arg_data
        self.arg_valor = arg_valor
        self.arg_tipo = arg_tipo
        self.tipo = self.arg_tipo.upper()
        self.referencia = arg_referencia
        self.data = datetime.datetime(int(self.arg_data[6:]),int(self.arg_data[3:5]),int(self.arg_data[:2]))
        self.ano = self.data.year
        self.mes = self.data.month
        self.dia = self.data.day
        if self.idioma == "ENG": self.valor=float(self.arg_valor)
        elif self.idioma == "PTB": self.valor=float(self.arg_valor.replace('.','').replace(',','.'))
        #Normalizando o tipo de dividendo
        #tipos existentes nas bases da Fundamentus:
        #"JUROS"
        #"JRS CAP PROPRIO"
        #"JRS CAP PRÓPRIO"
        #"DIVIDENDO"
        #"REST CAP DIN"
        #"JUROS MENSAL"
        #"RESTITUIÃ§Ã£O DE CAPITAL"
        #"DIVIDENDO MENSAL"
        #Fundamentus não apresenta dados de bonificações
        if self.tipo[:1] == "J": self.tipo_normalizado = "JCP"
        if self.tipo[:1] == "D": self.tipo_normalizado = "DIV"
        if self.tipo[:1] == "R": self.tipo_normalizado = "RST"



import datetime as _datetime

class timestamp:
    """
    horário da execução
    """
    def __init__(self):

        dtt_now = _datetime.datetime.now()
        self.str_yyyymmdd = dtt_now.strftime('%Y%m%d')
        self.str_hhmmss = dtt_now.strftime('%H%M%S')
        self.str_full = dtt_now.strftime('%Y-%m-%d %H:%M:%S')
        self.dtt_timestamp = dtt_now.timestamp()
        self.dtt_now = dtt_now


import datetime as _datetime
import zeep as _zeep
import xml.etree.ElementTree as _ET

class UltimaCotacaoDolar:

    # Documentação de apoio:
    #
    #   http://catalogo.governoeletronico.gov.br/arquivos/Documentos/WS_SGS_BCB.pdf - descrição do uso do Webservice
    #   http://python-zeep.readthedocs.io/en/master/ - zeep, biblioteca para trabalhar com webservices
    #   http://blog.tiagocrizanto.com/configuracoes-do-webservice-do-banco-central-cotacoes-diversas/

    def __init__(self):
        STR_WSDL='https://www3.bcb.gov.br/sgspub/JSP/sgsgeral/FachadaWSSGS.wsdl'
        # 07/11/19: nome da variável alterado de cliente para zzz.
        # por algum motivo, passou a dar erro depois do upgrade para 3.8.0
        try:
            OBJ_CLIENTE=_zeep.client.Client(wsdl=STR_WSDL)
            XML_RESPONSE = OBJ_CLIENTE.service.getUltimoValorVO(1).ultimoValor  # o 1 é o código da série temporal Dólar
            self.valor = XML_RESPONSE.valor._value_1
            self.data = _datetime.date(XML_RESPONSE.ano, XML_RESPONSE.mes, XML_RESPONSE.dia)
            self.erro = False
            self.erro_txt=''
        except Exception as e:
            self.valor = 0
            self.erro = True
            self.erro_txt = 'Ocorreu um erro: ' + e.args[0]
            dtt_now = _datetime.datetime.now()
            self.data = _datetime.date(dtt_now.year, dtt_now.month, dtt_now.day)

class CotacaoDolarData:

    def __init__(self, arg_data):
        self.data = _datetime.datetime.strptime(arg_data,'%d/%m/%Y')
        STR_WSDL = 'https://www3.bcb.gov.br/sgspub/JSP/sgsgeral/FachadaWSSGS.wsdl'
        try:
            OBJ_CLIENTE = _zeep.client.Client(wsdl=STR_WSDL)
            XML_RESPONSE = OBJ_CLIENTE.service.getValor(1,arg_data)  # o 1 é o código da série temporal Dólar
            self.valor = XML_RESPONSE._value_1
            self.erro = False
            self.erro_txt = ''
        except Exception as e:
            self.valor = 0
            self.erro = True
            self.erro_txt = e.args[0]
            if self.erro_txt.find('Value(s) not found') != -1:
                self.erro_txt = 'Não há cotação para ' + arg_data
            else:
                self.erro_txt = 'Ocorreu um erro: ' + _erro_txt

class CotacaoDolarHistorico:
    #ideias para chegar à solução vieram de vários sites:
    #https://python-zeep.readthedocs.io/en/master/datastructures.html
    #https://stackoverflow.com/questions/1130819/how-to-create-arraytype-for-wsdl-in-python-using-suds
    #depois fiquei testando no prompt até conseguir montar um objeto que funcionasse
    def __init__(self, arg_data_inicio, arg_data_fim):
        self.data_inicio = _datetime.datetime.strptime(arg_data_inicio, '%d/%m/%Y')
        self.data_fim = _datetime.datetime.strptime(arg_data_fim, '%d/%m/%Y')
        STR_WSDL = 'https://www3.bcb.gov.br/sgspub/JSP/sgsgeral/FachadaWSSGS.wsdl'
        try:
            OBJ_CLIENTE=_zeep.client.Client(wsdl=STR_WSDL)
            OBJ_FACTORY = OBJ_CLIENTE.type_factory('ns0')
            OBJ_ARRAYOFFLONG = OBJ_FACTORY.ArrayOfflong([1])
            XML_RESPONSE = OBJ_CLIENTE.service.getValoresSeriesXML(OBJ_ARRAYOFFLONG, arg_data_inicio, arg_data_fim)
            XML_ROOT = _ET.fromstring(XML_RESPONSE._value_1)
            lst_resultado = []
            for xml_element in XML_ROOT[0]:
                lst_resultado.append([xml_element[0].text, xml_element[1].text])
            self.erro = False
            self.erro_txt = ''
            self.cotacoes = lst_resultado
            self.itens = len(lst_resultado)
        except Exception as e:
            self.cotacoes = []
            self.itens = 0
            self.erro = True
            self.erro_txt = e.args[0]
            if self.erro_txt.find('Value(s) not found') != -1:
                self.erro_txt = 'Não há cotação para o período entre ' + arg_data_inicio + ' e ' + arg_data_fim
            else:
                self.erro_txt = 'Ocorreu um erro: ' + _erro_txt


import sys as _sys

def encoding():
    # define no encoding baseado no sistema operacional
    # arquivo em Windows (win32) tem que ser salvo em Unicode
    # se for Mac (darwin) tem que ser salvo como UTF-8
    str_platform = _sys.platform
    if str_platform == 'win32':
        return 'utf_16'
    elif str_platform == 'darwin':
        return 'utf_8'


class Color:
    __escape_code__ = '\033[{}m'

    #light fore colors
    red = __escape_code__.format(91)
    green = __escape_code__.format(92)
    yellow = __escape_code__.format(93)
    blue = __escape_code__.format(94)
    purple = __escape_code__.format(95)
    cyan = __escape_code__.format(96)
    white = __escape_code__.format(97)
    grey = __escape_code__.format(37)

    #dark fore colors
    dark_red = __escape_code__.format(31)
    dark_green = __escape_code__.format(32)
    dark_yellow = __escape_code__.format(33)
    dark_blue = __escape_code__.format(34)
    dark_purple = __escape_code__.format(35)
    dark_cyan = __escape_code__.format(36)

    #effects
    end = __escape_code__.format(0)
    bold = __escape_code__.format(1)
    italic = __escape_code__.format(3)
    underline = __escape_code__.format(4)
    strong_underline = __escape_code__.format(21)
    striketrough = __escape_code__.format(9)
    black_box = __escape_code__.format(51)

    #backgrounds
    black_background_white_front = __escape_code__.format(7)
    black_background = __escape_code__.format(40)
    dark_red_background = __escape_code__.format(41)
    dark_green_background = __escape_code__.format(42)
    dark_yellow_background = __escape_code__.format(43)
    dark_blue_background = __escape_code__.format(44)
    dark_purple_background = __escape_code__.format(45)
    dark_cyan_background = __escape_code__.format(46)
    dark_grey_background = __escape_code__.format(47)
    grey_background = __escape_code__.format(100)
    red_background = __escape_code__.format(101)
    green_background = __escape_code__.format(102)
    yellow_background = __escape_code__.format(103)
    blue_background = __escape_code__.format(104)
    purple_background = __escape_code__.format(105)
    cyan_background = __escape_code__.format(106)


class Header:

    header = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) '
                           'AppleWebKit/537.36 (KHTML, like Gecko) '
                           'Chrome/39.0.2171.95 Safari/537.36',
                           'Cache-Control': 'no-cache'}


import requests
from bs4 import BeautifulSoup
from datetime import date, datetime
from xlwt import Workbook, XFStyle
import configparser
import os
import sys
# para salvar o Excel em formato XLSX
# https://stackoverflow.com/questions/9918646/how-to-convert-xls-to-xlsx
import win32com.client as win32
import shutil
import progressbar  #https://gist.github.com/chrissimpkins/2c451bcff7bd7cbcf66e
import tqdm
# as funções auxiliares do MagicFormula
#sys.path.append(sys.path[0].replace("GetQuotes", "mftoolbox"))
#from ..mftoolbox.mftoolbox import *
from mftoolbox import mftoolbox
from constants import header

class cotacoes:

    def __init__(self, STR_LISTA_ATIVOS, STR_DATA_INICIO, STR_DATA_FIM):

        obj_dolar = mftoolbox.UltimaCotacaoDolar()
        STR_HEADER = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) '
                            'AppleWebKit/537.36 (KHTML, like Gecko) '
                            'Chrome/39.0.2171.95 Safari/537.36',
                    'Cache-Control': 'no-cache'}

        DTT_DATA_INICIO = datetime.strptime(STR_DATA_INICIO, '%d/%m/%Y')
        DTT_DATA_FIM = datetime.strptime(STR_DATA_FIM, '%d/%m/%Y')
        obj_dolar_historico = mftoolbox.CotacaoDolarHistorico(DTT_DATA_INICIO.strftime('%d/%m/%Y'), date.today().strftime('%d/%m/%Y'))
        STR_URL_COTACOES = 'https://www.ibovx.com.br/historico-papeis-bovespa.aspx?papel=**********&qtdpregoes=||||||||||'
        STR_URL_DETALHES_ATIVOS = 'http://www.fundamentus.com.br/detalhes.php?papel=**********'
        INT_BIG_CAP = FIL_CONFIG_INI.getint("Ativos", "Big Cap")
        INT_SMALL_CAP = FIL_CONFIG_INI.getint("Ativos", "Small Cap")
        STR_ATIVOS_MANDATORIOS = 'USD', 'IBOV'
        obj_dolar_historico = mftoolbox.CotacaoDolarHistorico(DTT_DATA_INICIO.strftime('%d/%m/%Y'),date.today().strftime('%d/%m/%Y'))
        DTT_NUMERO_PREGOES = datetime.today() - DTT_DATA_INICIO
        lst_acoes = STR_LISTA_ATIVOS.split(',')
        for item in STR_ATIVOS_MANDATORIOS:
            lst_acoes.append(item)  # Sempre inclui IBOV como padrão na lista
        str_linha = 1
        linha_ultimo_registro = 1

        for acao in lst_acoes:
            # -------------------- Início das ações da etapa --------------------
            lista_cells = []
            str_url_temp = STR_URL_COTACOES.replace('**********', acao).replace('||||||||||', str(DTT_NUMERO_PREGOES.days))
            # url = 'https://www.ibovx.com.br/historico-papeis-bovespa.aspx?papel=' +  acao + '&qtdpregoes=' + str(pregoes.days)
            r = requests.get(str_url_temp, headers=STR_HEADER)
            soup = BeautifulSoup(r.text, 'html.parser')

            # -------------------- Início da carga dos detalhes para cada ação, para achar o tipo de ação: Small, Mid e Big Cap
            str_tipo_capitalizacao = ""
            # if acao != "IBOV" and acao != "USD":
            if acao not in STR_ATIVOS_MANDATORIOS:
                rqt_site = requests.get(STR_URL_DETALHES_ATIVOS.replace('**********', acao), headers=STR_HEADER)
                html_soup = BeautifulSoup(rqt_site.text, 'html.parser')
                prs_data_detalhes_ativos = html_soup.find_all('td')

                flt_valor_mercado_reais = prs_data_detalhes_ativos[21].string  # mudança
                if flt_valor_mercado_reais is None or flt_valor_mercado_reais == '-':  # mudança
                    int_tem_dados_fundamentos = 0
                    flt_valor_mercado_reais = ''
                    flt_valor_mercado_usd = ''
                    str_tipo_capitalizacao = ''
                else:
                    flt_valor_mercado_reais = float(prs_data_detalhes_ativos[21].string.replace(".", "").replace(',', '.'))
                    flt_valor_mercado_usd = flt_valor_mercado_reais / FLT_DOLAR_HOJE
                    if flt_valor_mercado_usd >= INT_BIG_CAP:
                        str_tipo_capitalizacao = 'BIG CAP'
                    elif flt_valor_mercado_usd <= INT_SMALL_CAP:
                        str_tipo_capitalizacao = 'SMALL CAP'
                    else:
                        str_tipo_capitalizacao = 'MID CAP'

            int_registros = 1
            for td in soup.find_all('td'):
                int_registros += 1

            # valor máximo que a progressbar receberá = número de itens na lista que será iterada
            int_valor_maximo_barra = int_registros

            # inicializa a progressbar
            obj_bar = progressbar.ProgressBar(max_value=int_valor_maximo_barra, widgets=lst_widget_def)

            # define largura da barra
            obj_bar.term_width = INT_LARGURA_PROGRESSBAR

            # print('Coleta de dados de ', acao, ' ok!')

            counter = 1
            for td in soup.find_all('td'):
                lista_cells.append(td.text)
                # print(counter, td.text)

                str_novo_texto = str_raiz_mensagem_dinamica + acao.replace("'", "''")
                obj_bar.widgets[int_posicao_dinamico] = obj_bar.widgets[int_posicao_dinamico].replace(STR_DINAMICO_PROGRESSBAR,
                                                                                                      str_novo_texto)
                obj_bar.term_width = INT_LARGURA_PROGRESSBAR + len(str_novo_texto)

                obj_bar.update(counter)
                obj_bar.widgets[int_posicao_dinamico] = obj_bar.widgets[int_posicao_dinamico].replace(str_novo_texto,
                                                                                                      STR_DINAMICO_PROGRESSBAR)
                counter += 1

            int_len = len(lista_cells)
            y = 0
            if acao == "IBOV":
                dtt_ultimo_pregao = datetime.strptime("01/01/1900", '%d/%m/%Y')
                while y + 20 < int_len:
                    date = datetime.strptime(lista_cells[19 + y], '%d/%m/%Y')
                    # obj_dolar2 = mftoolbox.CotacaoDolarData(date.strftime("%d/%m/%Y"))
                    if dtt_ultimo_pregao < date:
                        dtt_ultimo_pregao = date
                    if date >= DTT_DATA_INICIO:
                        sheet1.write(str_linha, 1, acao)
                        sheet1.write(str_linha, 0, date, date_format)
                        sheet1.write(str_linha, 2, float(lista_cells[22 + y].replace(".", "").replace(",", ".")))
                        sheet1.write(str_linha, 3, float(lista_cells[20 + y].replace(".", "").replace(",", ".").replace("%", ""))/100)
                        sheet1.write(str_linha, 4, 0)
                        sheet1.write(str_linha, 5, 0)
                        str_linha = str_linha + 1
                    y = y + 7
            elif acao == "USD":
                for itens in obj_dolar_historico.cotacoes:
                    sheet1.write(str_linha, 1, acao)
                    sheet1.write(str_linha, 0, datetime.strptime(itens[0], "%d/%m/%Y"), date_format)
                    sheet1.write(str_linha, 2, float(itens[1]))
                    str_linha = str_linha + 1
            else:
                while y + 21 < int_len:
                    date = datetime.strptime(lista_cells[21 + y], '%d/%m/%Y')
                    if date >= DTT_DATA_INICIO:
                        sheet1.write(str_linha, 1, acao)
                        sheet1.write(str_linha, 0, datetime.strptime(lista_cells[21+y], "%d/%m/%Y"), date_format)
                        sheet1.write(str_linha, 2, float(lista_cells[24 + y].replace(".", "").replace(",", ".")))
                        sheet1.write(str_linha, 3, float(lista_cells[22 + y].replace(".", "").replace(",", ".").replace("%", ""))/100)
                        sheet1.write(str_linha, 4, lista_cells[28+y])
                        sheet1.write(str_linha, 5, lista_cells[29+y])
                        sheet1.write(str_linha, 6, str_tipo_capitalizacao)
                        str_linha = str_linha + 1
                    y = y + 9
            str_novo_texto = "Sucesso. Ativo " + acao + " carregado. Cotações = " + str(int(str_linha)-linha_ultimo_registro)
            linha_ultimo_registro = str_linha
            obj_bar.term_width = INT_LARGURA_PROGRESSBAR + len(str_novo_texto)
            obj_bar.widgets[int_posicao_dinamico] = obj_bar.widgets[int_posicao_dinamico].replace(STR_DINAMICO_PROGRESSBAR,
                                                                                                  str_novo_texto)
            obj_bar.finish()
            lst_widget_def[int_posicao_dinamico] = STR_DINAMICO_PROGRESSBAR
        print('')
        print('Data do último pregao: {}'.format(dtt_ultimo_pregao.strftime("%d/%m/%Y")))

        # -------------------- Fim das ações da etapa --------------------

    # apresenta a mensagem final de conclusão
    str_novo_texto = "Sucesso. Registros carregados = " + str(str_linha)
    obj_bar.term_width = INT_LARGURA_PROGRESSBAR + len(str_novo_texto)
    obj_bar.widgets[int_posicao_dinamico] = obj_bar.widgets[int_posicao_dinamico].replace(STR_DINAMICO_PROGRESSBAR,
                                                                                          str_novo_texto)
    obj_bar.finish()
    lst_widget_def[int_posicao_dinamico] = STR_DINAMICO_PROGRESSBAR
    # -------------------- Fim da coleta de dados das cotações --------------------

print('')
# print('Finalizando e salvando o XLS em '+STR_DIRETORIO)
dtt_lap_start = datetime.now()
try:
    os.remove(STR_DIRETORIO_SALVAR + STR_NOME_ARQUIVO)
except EFileNotFoundError:
    pass
try:
    os.remove(STR_DIRETORIO_SALVAR + STR_NOME_ARQUIVO + "x")
except FileNotFoundError:
    pass
try:
    wb.save(STR_DIRETORIO_SALVAR + STR_NOME_ARQUIVO)
except Exception as e:
    if os.path.exists(STR_DIRETORIO_SALVAR + STR_NOME_ARQUIVO):
        pass
    else:
        print('Alerta: não foi possível criar o arquivo ' + STR_DIRETORIO_SALVAR + STR_NOME_ARQUIVO)
        pass
excel = win32.DispatchEx("Excel.Application")
wb = excel.Workbooks.Open(STR_DIRETORIO_SALVAR + STR_NOME_ARQUIVO)
try:
    wb.SaveAs(STR_DIRETORIO_SALVAR + STR_NOME_ARQUIVO + "x", FileFormat=51)  # FileFormat = 51 is for .xlsx extension
except Exception as e:
    if os.path.exists(STR_DIRETORIO_SALVAR + STR_NOME_ARQUIVO + "x"):
        pass
    else:
        print('Alerta: não foi possível criar o arquivo ' + STR_DIRETORIO_SALVAR + STR_NOME_ARQUIVO + "x")
        pass
wb.Close()  # FileFormat = 56 is for .xls extension
excel.Application.Quit()

# wb.save(STR_DIRETORIO+STR_NOME_ARQUIVO)

dtt_lap = datetime.now() - dtt_lap_start
print('')
print('|■■■■■■■■■■■■■■■■■■| Elapsed Time: {}, Time: {}, Sucesso. Gravação do arquivo Excel. Cotações = {}'.format(str(dtt_lap).split('.')[0], str(dtt_lap).split('.')[0], len(lst_acoes)))
print('')
print('Tempo de execução (hh:mm:ss.ms) {}'.format(datetime.now() - obj_timestamp.dtt_now))
print('')
if BLN_PAUSA:
    input('Pressione Enter para finalizar...')

def ultimo_pregao(_ativo: 'código do ativo') -> 'texto com a data do último pregão para ' + _ativo + ' no formato DD/MM/AAAA':
    """

    :param _ativo: código do ativo
    :return: texto com a data do último pregão para o ativo no formato DD/MM/AAAA
    """

    url = 'https://www.ibovx.com.br/historico-papeis-bovespa.aspx?papel=' + ativo + '&qtdpregoes=1'
    r = requests.get(url, headers=mftoolbox.header.header)
    soup = BeautifulSoup(r.text, 'lxml')
    return soup.find_all('td')[19].text


import requests
from bs4 import BeautifulSoup
from datetime import datetime
import sys
#sys.path.append('C:\\Users\\coliveira\\OneDrive\\Coding\\Python\\MFToolbox\\')
from mftoolbox import constants, funcs
from selenium import webdriver
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
import time

def volatilidade(__ticker, **kwargs):
    """
        Busca no site da B3 a volatilidade histórica de um ativo

        :param __ticker: código do ativo
        :param __periodo = x, x em meses (1, 3, 6, 12). Padrão = 6 meses
        :return: tuple [ticker, nome de pregão, cotacao, horario, variação monetária, variação percentual
        """

    caps = DesiredCapabilities().CHROME
    caps["pageLoadStrategy"] = "eager"  # interactive
    options = webdriver.ChromeOptions()
    chrome_preferences = {'profile.managed_default_content_settings.images': 2}
    options.add_argument("headless")
    # noinspection SpellCheckingInspection
    options.add_experimental_option("prefs", chrome_preferences)
    browser = webdriver.Chrome(options=options, service_args=['--silent'], desired_capabilities=caps)

    try:
        __periodo = [int(kwargs.get('periodo'))]
        if '|1|3|6|12|'.find('|'+str([__periodo])+'|') == -1:
            __periodo = [6]
    except:
        __periodo = [1,3,6,12]
    __result = []
    for __item in __periodo:
        if __periodo == 1:
            __periodo_text = '12Meses'
        elif __periodo == 12:
            __periodo_text = '1Ano'
        else:
            __periodo_txt = str(__periodo) + 'Meses'
        # noinspection SpellCheckingInspection
        url = 'http://bvmf.bmfbovespa.com.br/cias-listadas/volatilidade-ativos/ResumoVolatilidadeAtivos.aspx?' \
              'metodo=padrao&' \
              'periodo=' + __periodo_txt + '&' \
              'codigo=' + __ticker + '&' \
              'idioma=pt'
        browser.get(url)
        html = browser.page_source
        soup = BeautifulSoup(html, "lxml")
        td_list = soup.find_all('td')
        try:
            __result.append([float(td_list[3].text.replace(",", ".")),float(td_list[4].text.replace(",", "."))])
        except:
            __result.append(['', ''])
    return __result



def ultima_cotacao(__ticker, **kwargs):
    """
    Busca no site Tradingview.com o nome de pregão de um ativo, a última cotação, seu horário,
    ganho monetário e percentual

    :param __ticker: código do ativo
    :param __delay = x, x em seundos
    :return: tuple [ticker, nome de pregão, cotacao, horario, variação monetária, variação percentual
    """
    try:
        __delay = int(kwargs.get('delay'))
    except:
        __delay = 0

    __url = 'https://www.tradingview.com/symbols/' + __ticker

    __caps = DesiredCapabilities().CHROME
    __caps["pageLoadStrategy"] = "eager"
    __options = webdriver.ChromeOptions()
    __chrome_preferences = {'profile.managed_default_content_settings.images': 2}
    __options.add_argument("headless")
    # noinspection SpellCheckingInspection
    __options.add_experimental_option("prefs", __chrome_preferences)
    __browser = webdriver.Chrome(options=__options, service_args=['--silent'], desired_capabilities=__caps)
    __browser.get(__url)
    time.sleep(__delay)
    __html = __browser.page_source
    __soup = BeautifulSoup(__html, "lxml")
    try:
        __nome_pregao = __soup.findAll("div", {"class": "tv-symbol-header__long-title-first-text"})[0].text
    except:
        __nome_pregao = ''
    try:
        __cotacao = float(__soup.findAll("div", {"class": "tv-symbol-price-quote__value js-symbol-last"})[0].text)
    except:
        __cotacao = 0
    try:
        __horario = __soup.findAll("span", {"class": "js-symbol-lp-time"})[0].text.replace(')','').replace('(','')
        __horario = datetime.strptime(__horario[:-6], '%b %d %H:%M')
    except:
        __horario = ''
    try:
        __variacao_monetaria = __soup.findAll("span",
                                          {"class": "js-symbol-change tv-symbol-price-quote__change-value"})[0].text
    except:
        __variacao_monetaria = 0
    try:
        __variacao_percentual = __soup.findAll("span",
                                       {"class": "js-symbol-change-pt tv-symbol-price-quote__change-value"})[0].text
    except:
        __variacao_percentual = 0

    return [__ticker, __nome_pregao, __cotacao, __horario, __variacao_monetaria, __variacao_percentual]


def ultimo_pregao(_ativo):
    """
    Busca no site IBOVX a data do último pregão para um ativo

    :param _ativo: código do ativo
    :return: tuple com a data como datetime, data no formato DD/MM/AAAA e a cotação
    """

    url = 'https://www.ibovx.com.br/historico-papeis-bovespa.aspx?papel=' + _ativo + '&qtdpregoes=1'
    r = requests.get(url, headers=constants.Header.header)
    soup = BeautifulSoup(r.text, 'lxml')
    _tabela = soup.find_all('td')
    try:
        if _tabela[20].text == 'Nº Negócios':
            posicao = 21
        else:
            posicao = 19
        return  (datetime.strptime(_tabela[posicao].text, '%d/%m/%Y'),_tabela[posicao].text)
    except:
        return (None, None)

def cotacao(_ativo, _data):
    """
    Busca no site IBOVX a cotação do ativo na data especificada. Retorna um tuple com data e cotação. Se não houver
    negociação naquela data, retorna a primeira cotação anterior disponível

    :param _ativo: código do ativo
    :param _data: data da cotação

    :return: tuple com a data como datetime, data no formato DD/MM/AAAA e a cotação
    """
    _data = datetime.strptime(_data, '%d/%m/%Y')
    _pregoes = str((datetime.now() - _data).days)
    url = 'https://www.ibovx.com.br/historico-papeis-bovespa.aspx?papel=' + _ativo + '&qtdpregoes=' + _pregoes
    r = requests.get(url, headers=constants.Header.header)
    soup = BeautifulSoup(r.text, 'lxml')
    _tabela = soup.find_all('td')
    i = len(_tabela)
    _cotacao_anterior = []
    if _tabela[20].text == 'Nº Negócios':
        incremento = 9
    else:
        incremento = 7

    try:
        while i >=0:
            _data_pagina = datetime.strptime(_tabela[i-incremento].text, '%d/%m/%Y')
            _cotacao = float(_tabela[i-incremento+3].text.replace('.','').replace(',','.'))
            if _data_pagina == _data:
                return (_data, _data.strftime('%d/%m/%Y'), _cotacao)
            elif _data_pagina > _data:
                return (_cotacao_anterior)
            _cotacao_anterior = (_data_pagina, _data_pagina.strftime('%d/%m/%Y'), _cotacao)
            i -= incremento
    except:
        return (_cotacao_anterior)


def cotacoes(_ativo, **kwargs):
    """
    Busca no site IBOVX a cotação do ativo na data especificada. Retorna um tuple com data e cotação. Se não houver
    negociação naquela data, retorna a primeira cotação anterior disponível

    :param _ativo: código do ativo
    :param data_inicio: data da primeira cotação, defautl = 01/01/2019
    :param pregoes: quantidade de pregoes retornados = 300

    :return: tuple com a data como datetime, data no formato DD/MM/AAAA e a cotação
    """
    _data = datetime.strptime(_data, '%d/%m/%Y')
    _pregoes = str((datetime.now() - _data).days)
    url = 'https://www.ibovx.com.br/historico-papeis-bovespa.aspx?papel=' + _ativo + '&qtdpregoes=' + _pregoes
    r = requests.get(url, headers=constants.Header.header)
    soup = BeautifulSoup(r.text, 'lxml')
    _tabela = soup.find_all('td')
    i = len(_tabela)
    _cotacao_anterior = []
    if _tabela[20].text == 'Nº Negócios':
        incremento = 9
    else:
        incremento = 7

    try:
        while i >=0:
            _data_pagina = datetime.strptime(_tabela[i-incremento].text, '%d/%m/%Y')
            _cotacao = float(_tabela[i-incremento+3].text.replace('.','').replace(',','.'))
            if _data_pagina == _data:
                return (_data, _data.strftime('%d/%m/%Y'), _cotacao)
            elif _data_pagina > _data:
                return (_cotacao_anterior)
            _cotacao_anterior = (_data_pagina, _data_pagina.strftime('%d/%m/%Y'), _cotacao)
            i -= incremento
    except:
        return (_cotacao_anterior)


def cotacoes_historicas(ativo, **kwargs):
    '''
    Carrega as cotações históricas de um ativo

    Args:
        ativo: ticker do ativo
        **kwargs:
            pregoes: quantidade de pregões retornados
            data_inicio: data do pregão mais antigo a ser retornado. Se não houve pregão nesta data, retorna o primeiro
                        pregão após esta data

    Returns:
        caso não seja passado o ticker do ativo (independentemente dos outros parâmetros):
            mensagem: Ativo não pode ser ''.
        caso o argumento data_inicio não seja uma data válida (e não tenha acontecido as situação acima):
            mensagem: Data 'DD/MM/YYYY' é inválida.
        se forem passados ambos os argumentos, vale o número de pregões
        se não for passado nenhum argumento, serão retornados os dados do último pregão

        lista de tuples com os seguintes dados:
            ativo
            data da cotação
            variação de preço percentual
            variação de preço em valor
            cotação
            preço de abertura
            preço mínimo
            preço máximo
            volume financeiro (ordem de grandeza)
            número de negócios


    '''

    if ativo == '':
        return "Ativo não pode ser ''"
    __param = {}
    for __item in kwargs:
        __param[__item.upper()] = kwargs[__item]
    try:
        pregoes = __param['PREGOES']
        __modo_pregoes = True
    except KeyError:
        # data_inicio = datetime.strptime('01/01/2019', '%d/%m/%Y')
        pregoes = 0
        __modo_pregoes = False
    try:
        data_inicio = __param['DATA_INICIO']
        data_inicio = datetime.strptime(data_inicio, '%d/%m/%Y')
    except (KeyError, ValueError):
        if not __modo_pregoes and len(__param) > 0:
            return "Data '" + data_inicio + "' é inválida."
        else:
            data_inicio = ''
            __modo_pregao = True

    if pregoes == 0 and data_inicio == '':
        pregoes = 1
        __modo_pregoes = True
    elif pregoes == 0 and data_inicio != '':
        pregoes = (datetime.now() - data_inicio).days
        __modo_pregoes = False

    __url = 'https://www.ibovx.com.br/historico-papeis-bovespa.aspx?papel=' + ativo + '&qtdpregoes=' + str(pregoes)
    __r = requests.get(__url, headers=constants.Header.header)
    __soup = BeautifulSoup(__r.text, 'lxml')
    if __soup.text.find('Papel não encontrado ou sem histórico.') >= 0:
        return "Ativo '" + ativo.upper() + "' não encontrado."
    __tabela = __soup.find_all('td')
    __i = len(__tabela)
    __cotacoes = []
    if __tabela[20].text == 'Nº Negócios':
        __incremento = 9
    else:
        __incremento = 7

    while __i >= 0:
        try:
            __data = datetime.strptime(__tabela[__i - __incremento + 0].text, '%d/%m/%Y')
        except ValueError:
            break
        __variacao_perc = funcs.num_ptb2us(__tabela[__i - __incremento + 1].text)
        __variacao_num = funcs.num_ptb2us(__tabela[__i - __incremento + 2].text)
        __cotacao = funcs.num_ptb2us(__tabela[__i - __incremento + 3].text)
        __abertura = funcs.num_ptb2us(__tabela[__i - __incremento + 4].text)
        __minimo = funcs.num_ptb2us(__tabela[__i - __incremento + 5].text)
        __maximo = funcs.num_ptb2us(__tabela[__i - __incremento + 6].text)
        __volumme = funcs.num_ptb2us(__tabela[__i - __incremento + 7].text)
        __negocios = funcs.num_ptb2us(__tabela[__i - __incremento + 8].text)
        if __tabela[__i - __incremento - 1].text.find('bannerresponsivoabaixomenu') > 0:
            __skip = 1
        else:
            __skip = 0
        __cotacoes.append((ativo, __data, __variacao_perc, __variacao_num, __cotacao, __abertura, __minimo, __maximo,
                           __volumme, __negocios))

        __i = __i - __incremento - __skip

    __cotacoes_final = []
    __cotacoes = sorted(__cotacoes, key=lambda x: x[1], reverse=True)
    # __cotacoes.sort(key=takeDate, reverse = True)
    if __modo_pregoes:
        __pregoes_carregados = len(__cotacoes)
        for __id, __linha in enumerate(__cotacoes):
            if __id < pregoes:
                __cotacoes_final.append(__linha)
    else:
        for __linha in __cotacoes:
            if __linha[1] >= data_inicio:
                __cotacoes_final.append(__linha)

    # return "Modo pregões = " + str(__modo_pregoes), "Quantidade de registros = " + str(len(__cotacoes_final)), "Parêmetro pregões = " + str(pregoes), "Última data dos registros = " + str(__cotacoes_final[len(__cotacoes_final)-1][1]), "Parâmetro data de início = " + str(data_inicio)
    return __cotacoes_final


import inspect
from mftoolbox.constants import Color as c

def error_stack(error, message):
    """
    Imprime mensagem de erro criada pelo usuário
    Args:
        context: contexto onde o erro foi gerado
        error: nome do erro
        message: mensagem explicativa do erro

    Returns: apenas imprime o tipo de erro, a mensagem explicativa e a lista de chamadas

    """
    context = inspect.getframeinfo(inspect.currentframe(), context=1), inspect.getouterframes(inspect.currentframe(),context=1)
    error = "Erro: " + error
    print(c.dark_red_background + c.bold + c.white + error + " " * (60 - len(error)) + c.end)
    print(message)
    print('Lista de chamadas:')
    last = False
    for id1, item1 in enumerate(context[1]):
        if last:
            break
        item2 = item1[0]
        filename, lineno, function, code_context, code = inspect.getframeinfo(item2)
        if function == "<module>":
            last = True
        if function not in ('get_context', '__init__', 'error_stack'):
            function = function.replace("<module>", "__main__")
            print("file: {}, line: {}, function: {}".format(filename, lineno, function))
    exit(1)

class RaiseError(Exception):
    """
    Levanta um erro definido pelo usuário
    """

    def __init__(self, RaisedError, message):
        """

        Args:
            RaisedError: Nome do erro
            message: mensagem explicativa do erro
        """
        error_stack(RaisedError, message)


import setuptools
from mftoolbox import __version__ as v

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="mftoolbox",
    version=v.__version__,
    author="Celso Oliveira",
    author_email="c.oliveira@live.com",
    description="A set of tools to support my MF2 and MagicFII projects",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/coliveira2001/mftoolbox",
    packages=['mftoolbox'], #setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    install_requires=[            # I get to this in a second
        'configparser',
        'zeep',
        'lxml',
        'bs4',
        'selenium',
        'progressbar',
        'tqdm'
      ],
    python_requires='>=3.5',
)


def num_ptb2us(str_number, **kwargs):
    """
    Trasnforma um número do formato brasileiro (0.000,00 ou 0.000,00%) para o formato americano (0,000.00).
    :param str_number: string com o número no formato brasileiro que será transformado
    :return: número (inteiro ou float) transformado. Se número for uma porcentagem, retorna o resultado dividido por 100
    """

    #if str_number == '' or str_number is None:
    #    raise Exception('NullValue', str_number)
    __param = {}
    for __item in kwargs:
        __param[__item.upper()] = kwargs[__item]
    try:
        __ignora_formato_us = __param['NUM_US']
    except KeyError:
        __ignora_formato_us = False
    try:
        __devolve_zero_em_erro = __param['DEVOLVE_ZERO']
    except KeyError:
        __devolve_zero_em_erro = True
    try:
        __devolve_erro = __param['DEVOLVE_ERRO']
    except KeyError:
        __devolve_erro = False
    try:
        if __ignora_formato_us:
            number_to_return = float(str_number.upper().replace('M', '000000').replace('K', '000').replace('B', '000000000').replace('R$ ','').replace('$','').replace(',', '').replace('\n', '').replace('%', '').strip())
        else:
            number_to_return = float(str_number.upper().replace('M', '000000').replace('K', '000').replace('B', '000000000').replace('R$ ','').replace('$','').replace('.', '').replace(',', '.').replace('\n', '').replace('%', '').strip())
        if str_number.find('%') != -1:
            number_to_return = number_to_return / 100
    except (ValueError, AttributeError):
        try:
            number_to_return = float(str_number)
        except:
            if __devolve_erro:
                number_to_return = "ERRO"
            elif __devolve_zero_em_erro:
                number_to_return = 0
            else:
                number_to_return = str_number
    return number_to_return

def clean_text(str_text):
    """
    Remove de uma string espaços em branco nas pontas e '\n'
    :param str_text: texto a ser formatado
    :return: texto formatado (se o parâmetro de origem era uma string), o próprio valor passado como parâmetro (se o parâmetro não for uma string) ou uma string vazia ('') se o parâmetro passado for nulo
    """

    if str_text is None:
        return ''
    try:
        text_to_return = str_text.replace('\n','').strip()
    except:
        text_to_return = str_text
    return text_to_return

def find_keys_by_val(_dict, _value):
    """
    Encontra o nome de uma chave em um dicionário a partir de seu valor.
    Referência: https://stackoverflow.com/questions/8023306/get-key-by-value-in-dictionary
    :param _dict: dicionário com chaves e valores
    :param _value: valor da chave
    :return: chave
    """
    try:
        key_2_return = list(_dict.keys())[list(_dict.values()).index(_value)]
    except:
        key_2_return = ''
    return key_2_return

def formata_fone(str_fone):
    """
    Formata no padrão brasileiro uma string contendo um telefone
    Se for um número celular, o formato final será +55 (DDD) #-####-####.
    Se for um número fixo, o formato final será +55 (DDD) ####-####.
    :param str_fone: string contendo um número de telefone
    :return: string formatada com o número de telefone no padrão brasileiro
    """
    str_fone = str_fone.strip().replace(' ', '').replace('-', '').replace('+','').replace('.','').replace('(','').replace(')','')
    if len(str_fone) == 8: #assumindo que é um número fixo sem DDD e DDI
        str_fone = '+55 (11) ' + str_fone[:4] + '-' + str_fone[4:]
    elif len(str_fone) == 9: #assumindo que é um número de celular sem DDD e DDI
        str_fone = '+55 (11) ' + str_fone[:5] + '-' + str_fone[5:]
    elif len(str_fone) == 10: #assumindo que é um número fixo com DDD e sem DDI
        str_fone = '+55 (' + str_fone[:2] + ') ' + str_fone[2:6] + '-' + str_fone[6:]
    elif len(str_fone) == 11: #assumindo que é um número de celular com DDD e sem DDI
        str_fone = '+55 (' + str_fone[:2] + ') ' + str_fone[2:3] + '-' + str_fone[3:7] + '-' + str_fone[7:]
    elif len(str_fone) == 12:  # assumindo que é um número fixo com DDD e DDI
        str_fone = '+' + str_fone[:2] + ' (' + str_fone[2:4] + ') ' + str_fone[4:8] + '-' + str_fone[8:12]
    elif len(str_fone) == 13:  # assumindo que é um número celular com DDD e DDI
        str_fone = '+' + str_fone[:2] + ' (' + str_fone[2:4] + ') ' + str_fone[4:5] + '-' + str_fone[5:9] + '-' + str_fone[9:13]
    return str_fone

def formata_url(str_url):
    """
    Formata no padrão https://url uma string contendo uma url
    :param str_url: url
    :return: url formatada
    """
    str_url = str_url.lower().replace('https','').replace('http','').replace('://','')
    return 'https://' + str_url

def formata_email(str_email):
    """
    Formata email no padrão lestras minúsculas e garante que tenho um '@'. ainda não valida formato dos domínios.
    :param str_email: email
    :return: email formatado
    """
    if str_email.find('@') < 0:
        return 'email com formato errado. Faltou \'@\''
    else:
        return clean_text(str_email).lower()

def formata_cnpj(str_cnpj):
    """
    Formata uma string contendo um CNPJ. Não checa se é um CNPJ válido.
    :param str_cnpj: string com o CNPJ
    :return: string formatada. Se a string não puder ser formatada (quantidade menor de dígitos), retorna mensagem de erro: 'CNPJ 'zzzzz' não pode ser formatado', onde zzzzzz é a string passada como parâmetro.
    """
    _temp_cnpj = clean_text(str_cnpj).replace('.', '').replace('/', '').replace('-', '')
    if not _temp_cnpj.isdigit() or len(_temp_cnpj) != 14:
        return 'CNPJ \'' + str_cnpj + '\' não pode ser formatado'
    else:
        return _temp_cnpj[:2] + '.' + _temp_cnpj[2:5] + '.' + _temp_cnpj[5:8] + '/' + _temp_cnpj[8:12] + '-' + _temp_cnpj[12:14]

class MesPtb(object):

    __dic_meses = {
        'JANEIRO': '01',
        'FEVEREIRO': '02',
        'MARÇO': '03',
        'ABRIL': '04',
        'MAIO': '05',
        'JUNHO': '06',
        'JULHO': '07',
        'AGOSTO': '08',
        'SETEMBRO': '09',
        'OUTUBRO': '10',
        'NOVEMBRO': '11',
        'DEZEMBRO': '12'}

    def __init__(self):

        self.__txt_mes_ptb = ''
        self.__num_mes_ptb = 0

    @property
    def txt_mes_ptb(self):
        return self.__txt_mes_ptb

    @txt_mes_ptb.setter
    def txt_mes_ptb(self, value):
        self.__txt_mes_ptb = value.upper().strip()
        try:
            self.__num_mes_ptb = self.__dic_meses[self.__txt_mes_ptb]
        except KeyError:
            self.__num_mes_ptb = 0
            self.__txt_mes_ptb = 'MES INEXISTENTE'

    @property
    def num_mes_ptb(self):
        return self.__num_mes_ptb

    @num_mes_ptb.setter
    def num_mes_ptb(self, value):

        if type(value) is int:
            value = str(value).zfill(2)
        elif type(value) is str:
            value = int(value)
            if value > 12 or value <1:
                value = 0
            else:
                value = str(value).zfill(2)
        else:
            value = 0
        try:
            self.__txt_mes_ptb = list(self.__dic_meses.keys())[list(self.__dic_meses.values()).index(value)]
            self.__num_mes_ptb = value
        except KeyError:
            self.__num_mes_ptb = 0
            self.__txt_mes_ptb = 'MES INEXISTENTE'

def error_stack(context, error, message):
    """
    Imprime mensagem de erro criada pelo usuário
    Args:
        context: contexto onde o erro foi gerado
        error: nome do erro
        message: mensagem explicativa do erro

    Returns: apenas imprime o tipo de erro, a mensagem explicativa e a lista de chamadas

    """
    context = inspect.getframeinfo(inspect.currentframe(), context=1), inspect.getouterframes(inspect.currentframe(),context=1)
    error = "Erro: " + error
    print(Color.dark_red_background + Color.bold + Color.white + error + " " * (60 - len(error)) + Color.end)
    print(message)
    print('Lista de chamadas:')
    last = False
    for id1, item1 in enumerate(context[1]):
        if last:
            break
        item2 = item1[0]
        filename, lineno, function, code_context, code = inspect.getframeinfo(item2)
        if function == "<module>":
            last = True
        if function not in ('get_context', '__init__', 'error_stack'):
            function = function.replace("<module>", "__main__")
            print("file: {}, line: {}, function: {}".format(filename, lineno, function))

class RaiseError(Exception):
    """
    Levanta um erro definido pelo usuário
    """

    def __init__(self, RaisedError, message):
        """

        Args:
            RaisedError: Nome do erro
            message: mensagem explicativa do erro
        """
        error_stack(RaisedError, message)


import sys
sys.path.append("..") # Adds higher directory to python modules path.
import version__ as v
#__version__ = "4.0.14"
__version__ = v.__version__
__version_info__ = tuple([ int(num) for num in __version__.split('.')])